<?php
// $Id: localizer.inc,v 1.1.2.2 2007/04/24 20:55:01 sun Exp $

/**
 * Returns database definitions for a given object type.
 * 
 * @param string $object
 *   An object name, 'node' or 'translation'.
 * 
 * @return array
 *   An array containing table name and key of the object, optionally including
 *   a parent key and additional table columns.
 */
function localizer_get_object_definition(&$object) {
  // If $object is an array, it contains the definition already.
  if (is_array($object)) {
    return;
  }
  switch ($object) {
    case 'node':
      $object = array(
        'table' => 'localizernode',
        'key' => 'nid',
        'parentkey' => 'pid',
        'locale' => 'locale',
        'any' => 'any',
      );
      return;
    
    case 'translation':
      $object = array(
        'table' => 'localizertranslation',
        'key' => 'tid',
        'object_name' => 'object_name',
        'object_key' => 'object_key',
        'object_field' => 'object_field',
        'translation' => 'translation',
        'locale' => 'locale',
      );
      return;
    
    case 'user':
      $object = array(
        'table' => 'localizeruser',
        'key' => 'uid',
        'contentlocales' => 'contentlocales',
        'switch_uicontent' => 'switch_uicontent',
      );
      return;
  }
}

/**
 * Find a content translation in translation index.
 * 
 * @param string $object
 *   The object name to search for.
 * @param string $conditions
 *   A SQL where clause to search for.
 * @param bool $all
 *   Whether to return one or all query result rows.
 * @param bool $force
 *   Whether to return a cached query result if one exists.
 * 
 * @return array
 *   An array containing the query result(s).
 */
function localizer_find($object, $conditions = NULL, $all = TRUE, $force = FALSE) {
  static $localcache = array();
  
  $cid = $conditions .':'. ($all ? 'all' : 'one');
  if (isset($localcache[$cid]) && !$force) {
    return $localcache[$cid];
  }
  
  localizer_get_object_definition($object);
  
  $sql = 'SELECT * FROM {'. $object['table'] .'}';
  if ($conditions) {
    $sql .= ' WHERE '. $conditions;
  }
  if (!$all) {
    $result = db_query_range($sql, 0, 1);
  }
  else {
    $result = db_query($sql);
  }
  
  // Parse database results into object specific result sets.
  unset($object['table']);
  $key = $object['key'];
  $items = array();
  while ($item = db_fetch_array($result)) {
    foreach ($object as $column) {
      $items[$item[$key]][$column] = $item[$column];
    }
  }
  
  if (!$all) {
    $oneitem = array();
    foreach ($items as $object['key'] => $item) {
      foreach ($item as $key => $value) {
        $oneitem[$key] = $value;
      }
      break;
    }
    $localecache[$cid] = $oneitem;
    return $oneitem;
  }
  else {
    $localecache[$cid] = $items;
    return $items;
  }
}

/**
 * Fetch a single content from translation index.
 */
function localizer_findbyid($object, $id, $force = FALSE) {
  localizer_get_object_definition($object);
  return localizer_find($object, $object['key'] ." = '$id'", FALSE, $force);
}

/**
 * Save a content to translation index.
 */
function localizer_save($object, $item) {
  if (!is_array($item)) {
    return FALSE;
  }
  localizer_get_object_definition($object);

  if ($object['table'] == 'localizertranslation' && !$item[$object['key']]) {
    $translation = localizer_find('translation', "object_name='". $item['object_name'] ."' AND object_key='". $item['object_key'] ."' AND object_field='". $item['object_field'] ."' AND locale='". $item['locale'] ."'", FALSE);
    $item[$object['key']] = $translation[$object['key']];
  }
  
  if (db_result(db_query("SELECT COUNT(%s) FROM {%s} WHERE %s = %d", $object['key'], $object['table'], $object['key'], $item[$object['key']])) > 0) {
    return localizer_update($object, $item);
  }
  else {
    if (isset($object['parentkey']) && !$item[$object['parentkey']]) {
      // If no source content id is given, set source content id to current
      // content id.
      $item[$object['parentkey']] = $item[$object['key']];
    }
    return localizer_insert($object, $item);
  }
}

/**
 * Insert a new content into translation index.
 */
function localizer_insert($object, $item) {
  if (!is_array($item)) {
    return FALSE;
  }
  localizer_get_object_definition($object);
  
  if ($object['table'] == 'localizertranslation') {
    $item[$object['key']] = db_next_id('{localizertranslation}_'. $object['key']);
  }
  
  $table = $object['table'];
  unset($object['table']);
  
  $fields = array();
  foreach ($object as $key => $column) {
    $fields['columns'][] = $column;
    if ($key == 'key' || $key == 'parentkey') {
      $fields['types'][] = '%d';
    }
    else {
      $fields['types'][] = "'%s'";
    }
    $fields['values'][] = $item[$column];
  }
  db_query("INSERT INTO {$table} (". implode(',', $fields['columns']) .") VALUES (". implode(',', $fields['types']) .")", $fields['values']);
  return $item[$object['key']];
}

/**
 * Update a content in translation index.
 */
function localizer_update($object, $item) {
  if (!is_array($item)) {
    return FALSE;
  }
  localizer_get_object_definition($object);
  
  $table = $object['table'];
  unset($object['table']);
  
  $fields = array();
  $where  = array();
  foreach ($object as $key => $column) {
    if ($key == 'key') {
      $fields['where'] = $column .' = '. $item[$column];
      continue;
    }
    if ($key == 'parentkey') {
      $fields['assigns'][] = $column .' = %d';
    }
    else {
      $fields['assigns'][] = $column ." = '%s'";
    }
    $fields['values'][] = $item[$column];
  }
  return db_query("UPDATE {$table} SET ". implode(',', $fields['assigns']) ." WHERE ". $fields['where'], $fields['values']);
}

/**
 * Delete a translation reference in translation index.
 *
 * @todo If a source content is deleted, all translations of this node have to
 *       be deleted, too.
 */
function localizer_delete($object, $id) {
  if ($id) {
    localizer_get_object_definition($object);
    db_query('DELETE FROM {%s} WHERE %s = %d', $object['table'], $object['key'], $id);
  }
}

/**
 * Delete all translations for a given condition.
 */
function localizer_delete_all($object, $conditions) {
  if ($conditions) {
    localizer_get_object_definition($object);
    db_query('DELETE FROM {%s} WHERE '. $conditions, $object['table']);
  }
}

